/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file MyDebug.h
\brief Partie cliente du debugeur MyDebugServer
*/

#ifndef INCLUDE_MY_DEBUG_H
#define INCLUDE_MY_DEBUG_H

#include <windows.h>
#include <stdio.h>
#include <stdarg.h>
#include "MyString.h"

/*!
\brief MyDebug est la partie cliente (qui envoi) du debugeur (cf MyServerDebug)

Protocole client<->serveur:
header "tabName" text
header = {#line,#event}

\code
Ex.:
MyDebug::SendEvent("mon_programme","mon_evenement(%d,%d)",param1,param2);
\endcode
*/
class MyDebug  
{
	public:
	static HWND hWndServer;

	static BOOL Send(char *message)
	{
		COPYDATASTRUCT cpd;
		cpd.dwData = 0;
		cpd.cbData = (DWORD)strlen(message)+1;
		cpd.lpData = (void *)message;
		return (BOOL)SendMessage(MyDebug::hWndServer, WM_COPYDATA, NULL,(LPARAM)&cpd);
	}

public:
	MyDebug()
	{
		Start();
	}

	~MyDebug()
	{
	}

	static void Start()
	{
		if (!hWndServer)
		{
			FindServer();
		}
	}

	static BOOL FindServer()
	{
		hWndServer = FindWindow(NULL,"DebugServer");
		return hWndServer?TRUE:FALSE;
	}

	static BOOL SendEvent(char *tabName,char *message,...)
	{
		MyString buffer;

		char temp[1024];
		va_list args;
		va_start(args,message);
		vsprintf(temp,message,args);

		buffer<<"#event \""<<tabName<<"\" "<<temp<<"\n";
		return Send(buffer);
	}

	static BOOL SendLine(char *tabName,char *message,...)
	{
		MyString buffer;

		char temp[1024];
		va_list args;
		va_start(args,message);
		vsprintf(temp,message,args);

		buffer<<"#line \""<<tabName<<"\" "<<temp<<"\n";
		return Send(buffer);
	}
};

#endif 

