#include <Windows.h>
#include <wingdi.h>
#include <stdio.h>

#include "myListView.h"

#include "ClientDebug.h"
#include "utilsDebug.h"

CClientDebug debugClient;

LRESULT ListViewCallBack (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam,ULONG ptrContext,BOOL *isEnd)
{
	CMyListView *listView = (CMyListView *) ptrContext;
	*isEnd=FALSE;

	switch (message)
	{
	case WM_SIZE:
		if (listView->m_tree) listView->TreeColumnSizeChanged();
		break;

	case WM_HSCROLL:
		//listView->InvalidateRect(); // je sais pas pourkoi y a besoin de ca !!
		break;

	case WM_VSCROLL:
		if (listView->m_tree) 
		{
			int nScrollCode = (int) LOWORD(wParam); // scroll bar value 
			int nPos = (short int) HIWORD(wParam);  // scroll box position 
			listView->m_tree->scroll.SetVert(nScrollCode,nPos);
		}
		// a faire
		//SendMessage(listView->GetParent(),WM_COMMAND,listView->GetControlID(),ListVScroll);
		break;

	case WM_KILLFOCUS:
		if (listView->m_tree)
		{
			COLORREF textColor,backgroundColor;
			listView->GetHightLightColorUnFocus(&textColor,&backgroundColor);
			listView->m_tree->SetHightLightColorUnFocus(textColor,backgroundColor);
		}
		break;

	case WM_SETFOCUS: 
		if (listView->m_tree)
		{
			COLORREF textColor,backgroundColor;
			listView->GetHightLightColor(&textColor,&backgroundColor);
			listView->m_tree->SetHightLightColorUnFocus(textColor,backgroundColor);
			//listView->SetLineSelected(-1);
			//int lineSel = listView->GetLineSelected();
			//listView->SetLineSelected(lineSel+1);
		}	
		break;

	case WM_MOUSEMOVE:
		//if (!listView->IsFocus()) listView->SetFocus();

		LVHITTESTINFO ht;
		ht.pt.x = LOWORD(lParam);
		ht.pt.y = HIWORD(lParam); 
		ListView_HitTest(hDlg,&ht);
		ListView_SubItemHitTest(hDlg,&ht);
		if (ht.iItem!=listView->GetLineHover() || ht.iSubItem!=listView->GetColumnHover())
		{
			listView->SetLineHover(ht.iItem);
			listView->SetColumnHover(ht.iSubItem);
			// a faire
			//SendMessage(listView->GetParent(),WM_COMMAND,listView->GetControlID(),ListHoverChanged);
		}
		break;
	
	}
	return 0;
}

// ******************************************************************************

LRESULT ListViewParentCallBack (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam,ULONG ptrContext,BOOL *isEnd)
{
	CMyListView *listView = (CMyListView *) ptrContext;
	*isEnd=FALSE;
	switch (message)
	{ 
	case WM_NOTIFY:
		{
			int result;
			*isEnd = listView->Notify(lParam,&result);
			if (*isEnd) return result; 
		}
		break;

	case WM_MEASUREITEM:
		if ((int)wParam==listView->GetControlID())
		{
			if (listView->GetItemHeight()==-1) break;
			LPMEASUREITEMSTRUCT ptrInfo = (LPMEASUREITEMSTRUCT)lParam;
			ptrInfo->itemHeight = listView->GetItemHeight(); 
			*isEnd=TRUE;
		}
		break;
	}
	return 0;
}

/* *****************************************************************************
***************************************************************************** */

void CMyListView::Init()
{
	classNotify=this;
	m_tree = NULL;
	m_nbColumn = 0;
	m_nbCount = 0;
	m_lineHover = -1;
	m_columnHover = -1;

	oldColumn = -1; 
	oldWidth = -1;
	isRepaint = FALSE;

	SetLineColor(GetSysColor(COLOR_WINDOWTEXT),GetSysColor(COLOR_WINDOW));
	SetHightLightColor(GetSysColor(COLOR_HIGHLIGHTTEXT),GetSysColor(COLOR_HIGHLIGHT));
	SetHightLightColorUnFocus(GetSysColor(COLOR_HIGHLIGHTTEXT),GetSysColor(COLOR_INACTIVECAPTION));

	SetFullRawSelect(TRUE);

	subParent.Set(ListViewParentCallBack,(LONG)this);
	sub.Set(ListViewCallBack,(LONG)this);
}

/* *****************************************************************************
***************************************************************************** */

CMyListView::CMyListView (HWND hWndListView)
{
	*this=(hWndListView);
}

CMyListView::CMyListView()
{
}

void CMyListView::operator=(HWND hWndListView)
{
	CMyControl *control = (CMyControl *)this;
	*control = hWndListView;	
	m_controlID = GetDlgCtrlID();
	Init();
	classNotify->hWnd = hWndListView;
}

void CMyListView::operator=(CMyListViewNotify *ptrClassNotify)
{
	classNotify = ptrClassNotify;
}

/* *****************************************************************************
***************************************************************************** */

CMyListView::CMyListView(HWND hwndMaster,int x,int y,int dx,int dy,int optionalControlID) 
: CMyControl(hwndMaster)
{
	InitCommonControls();
	
	m_controlID = optionalControlID;

	DWORD dwStyle = WS_TABSTOP | WS_CHILD | WS_BORDER | WS_VISIBLE |LVS_AUTOARRANGE |LVS_REPORT | LVS_OWNERDATA | LVS_EX_FULLROWSELECT;
	m_hWndControl = CreateWindowEx( 
		WS_EX_CLIENTEDGE,WC_LISTVIEW,"",dwStyle,
		x,y,dx,dy,
		hwndMaster,NULL,GethInstance(),NULL);

	if(m_hWndControl) ListView_DeleteAllItems(m_hWndControl);  
	Init();
}

/* *****************************************************************************
***************************************************************************** */

CMyListView::~CMyListView()
{
}

/* *****************************************************************************
***************************************************************************** */

BOOL CMyListView::AddColumnPercent (char *nameHead,double percent)
{
	int dx;
	GetSize(&dx,NULL);
	return AddColumn(nameHead,(int)(dx*percent));
}

/* *****************************************************************************
***************************************************************************** */

BOOL CMyListView::AddColumn (char *nameHead,int size)
{
	LV_COLUMN   lvColumn;

	//initialize the columns
	lvColumn.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
	lvColumn.fmt = LVCFMT_LEFT;
	lvColumn.cx = size;
	lvColumn.pszText = nameHead;
	return ListView_InsertColumn(m_hWndControl, m_nbColumn++, &lvColumn) == m_nbColumn-1;
}

BOOL CMyListView::SetColumn	(long index,char *nameHead)
{
	LV_COLUMN   lvColumn;

	//initialize the columns
	lvColumn.mask = LVCF_FMT | LVCF_TEXT | LVCF_SUBITEM;
	lvColumn.fmt = LVCFMT_LEFT;
	lvColumn.pszText = nameHead;
	return ListView_SetColumn(m_hWndControl, index, &lvColumn);
}

/* *****************************************************************************
***************************************************************************** */

void CMyListView::SetGridLinesView(BOOL active)
{
	ListView_SetExtendedListViewStyleEx(m_hWndControl,LVS_EX_GRIDLINES,active?LVS_EX_GRIDLINES:0); 
}


/* *****************************************************************************
***************************************************************************** */

void CMyListView::SetNbCount(int nbCountx)
{
	m_nbCount = nbCountx;
	ListView_SetItemCount(m_hWndControl, m_nbCount);
}

/* *************************************************************************
	interprete le message WM_NOTIFY de la callback mere du ListCtrl:

	getDispInfo     iItem iSubItem; 
	itemChanged		iItem

	char temp[128];
	CMyListView list;

	ENotify res = list.Notify(lParam);
	switch (res)
	{
		case getDispInfo:
			sprintf(temp, "On demande les infos concernant: ligne %d - colone %d", list.GetLine , list.GetColumn);
			list.SetItemText(temp);
			break;

		case itemChanged:
			sprintf(temp,"ligne: %d colone %d",list.GetLineSelected ,list.GetColumnSelected);
			MessageBox(NULL,temp,"Nouvelle selection:",0);
			break;
	}
************************************************************************* */

BOOL CMyListView::Notify (LPARAM lParam,int *result)
{
	LPNMHDR lpnmh = (LPNMHDR) lParam;

	*result = 0;

	if (lpnmh->hwndFrom == GetHeader())
	{
		switch (lpnmh->code)
		{
			case HDN_ITEMCHANGED: //  Indicates that the attributes of an item are about to change.  
				{
					LPNMHEADER phdr = (LPNMHEADER) lParam;
					if (oldColumn==phdr->iItem && oldWidth==GetColumnWidth(phdr->iItem)) break;
					oldColumn =phdr->iItem;
					oldWidth=GetColumnWidth(phdr->iItem);
					InvalidateRect();
					if (m_tree) TreeColumnSizeChanged();
					classNotify->OnColumnSizeChanged(oldColumn,oldWidth);
					return TRUE;
				}
/*
			case HDN_BEGINTRACK:  //Signals the start of divider dragging.  
			case HDN_DIVIDERDBLCLICK: //  Indicates that the user double-clicked a divider. 
			case HDN_ENDTRACK:  //Signals the end of divider dragging.  
			case HDN_ITEMCHANGED: //  Indicates a change in the attributes of an item. 
			case HDN_ITEMCLICK: //  Indicates that the user clicked an item.  
			case HDN_ITEMDBLCLICK:  //Indicates that the user double-clicked an item. 
			case HDN_TRACK:  // Indicates that the user dragged a divider. 
				break;
*/
		}
		//return eNotify;
	}
	
	if (lpnmh->hwndFrom != m_hWndControl) return FALSE;

	switch (lpnmh->code)
	{
		case LVN_GETDISPINFO:
			{
				LV_DISPINFO *lpdi = (LV_DISPINFO *)lParam;
				
				if ( (lpdi->item.mask&LVIF_TEXT) == LVIF_TEXT)
				{
					classNotify->OnGetText(lpdi->item.iItem,lpdi->item.iSubItem,lpdi->item.pszText);
					return TRUE;
				}
			}
/*
		case LVN_GETINFOTIP: // uniquement en mode ICON view
			{
				LPNMLVGETINFOTIP pGetInfoTip = (LPNMLVGETINFOTIP)lParam;
				m_line = pGetInfoTip->iItem;
				m_column = pGetInfoTip->iSubItem;
				m_itemText = pGetInfoTip->pszText;
				eNotify = ListGetInfoTip;
				break;
			}
*/
			
		case LVN_ITEMCHANGED:
			{
				LPNMLISTVIEW lpnmlist = (LPNMLISTVIEW) lParam;
				if (lpnmlist->uNewState==(LVIS_SELECTED|LVIS_FOCUSED))
				{
					m_lineSelected = lpnmlist->iItem;
					m_columnSelected = lpnmlist->iSubItem;
					if (m_tree)
					{
						RECT rect;
						ListView_GetItemRect(GetHWND(),GetLineSelected(),&rect,LVIR_BOUNDS);
						m_tree->InvalidateRect();
						m_tree->SetCurrentItem(m_tree->GetItem(10,rect.top-5));
						//SetLineSelected(m_lineSelected);
					}
					classNotify->OnItemChanged(m_lineSelected,m_columnSelected);
					return TRUE;
				}
				break;
			}
/*
		case NM_DBLCLK:
			eNotify = ListDblClick;
			break;
		*/	
		case LVN_KEYDOWN:
			{
				LPNMLVKEYDOWN lpKey = (LPNMLVKEYDOWN) lParam;
				classNotify->OnKeyDown(lpKey->wVKey);
				return TRUE;
			}
/*
		case NM_CUSTOMDRAW: // changement de la taille d'un colone
		    *result = CDRF_DODEFAULT;
			//eNotify= ListCustomDraw;

			LPNMLVCUSTOMDRAW cd = (LPNMLVCUSTOMDRAW)lParam;
			int iRow = cd->nmcd.dwItemSpec;
			int iColumn = cd->nmcd.uItemState;
			
			CMyString temp,temp2;
			temp<<"iRow:"<<iRow<<" iColumn:"<<iColumn<<" isRepaint:"<<isRepaint<< " stage:"<<(int)cd->nmcd.dwDrawStage;

			switch (cd->nmcd.dwDrawStage)
			{
				case CDDS_PREPAINT:
					  if (isRepaint) 
					  {
						  temp2<<"prepaint IS_REPAINT "<<temp;
						  debugClient.SendEvent("CustomDraw",temp2);
						  isRepaint = FALSE;
						  *result = CDRF_SKIPDEFAULT;
					  }
					  else 
					  {
						  temp2<<"prepaint NOT IS_REPAINT "<<temp;
						  debugClient.SendEvent("CustomDraw",temp2);
						  if (m_tree && iColumn==0) 
						  {
							  // pour optimisation
							  //RECT rect;
							  //ListView_GetItemRect(GetHWND(),iRow,&rect,LVIR_BOUNDS);
							  //m_tree->GetPos((int*)&rect.left,(int *)&rect.top);
							  //int dx;
							  //m_tree->GetSize(&dx,NULL);
							  //rect.right=rect.left+dx;
							  //rect.bottom=rect.top+m_tree->GetItemHeight();
							  m_tree->InvalidateRect(NULL); //&rect)
							 
						  }
						  *result=CDRF_NOTIFYITEMDRAW;
					  }
					  return TRUE;

				case CDDS_POSTPAINT:
					temp2<<"postpaint "<<temp;
					debugClient.SendEvent("CustomDraw",temp2);
					if (m_tree) m_tree->InvalidateRect();
					return TRUE;

				case CDDS_ITEMPREPAINT:
					  if (!IsLineSelected(iRow))
					  {
						  temp2<<"item prepaint NOT IS_LINE_SEL "<<temp;
						  debugClient.SendEvent("CustomDraw",temp2);
						  cd->clrText   = m_textColor; // Use my foreground color
						  cd->clrTextBk = m_backgroundColor; // Use my background color
					  }
					  else
					  {
						  temp2<<"item prepaint IS_LINE_SEL "<<temp;
						  debugClient.SendEvent("CustomDraw",temp2);
						  EnableLineSelected(iRow,FALSE);
						  BOOL isFocus = cd->nmcd.uItemState&CDIS_FOCUS;
						  cd->clrText   = isFocus?m_textColorHi:m_textColorHiUf; // Use my foreground hilite color
						  cd->clrTextBk = isFocus?m_backgroundColorHi:m_backgroundColorHiUf; // Use my background hilite color
						  //EnableLineSelected(iRow,TRUE);
						  *result = CDRF_NOTIFYPOSTPAINT;
						  isRepaint=TRUE;
					  }
					  return TRUE;

				case CDDS_ITEMPOSTPAINT:
					temp2<<"item postpaint "<<temp;
					debugClient.SendEvent("CustomDraw",temp2);
					EnableLineSelected(iRow,TRUE);
					return TRUE;
				default:
					temp2<<"DEFAULT"<<temp;
					debugClient.SendEvent("CustomDraw",temp2);

			}
			break;		
*/
		case NM_CUSTOMDRAW: // changement de la taille d'un colone
			static BOOL bIsHighlighted = FALSE;

			*result = CDRF_DODEFAULT;
			LPNMLVCUSTOMDRAW cd = (LPNMLVCUSTOMDRAW)lParam;

			CMyString temp,temp2;
			//temp<<"iRow:"<<iRow<<" iColumn:"<<iColumn<<" isRepaint:"<<isRepaint<< " stage:"<<(int)cd->nmcd.dwDrawStage;
			int iRow;

			switch (cd->nmcd.dwDrawStage)
			{
			case CDDS_PREPAINT:
				if (isRepaint) 
				{
					temp2<<"prepaint IS_REPAINT "<<temp;
					debugClient.SendEvent("CustomDraw",temp2);
					isRepaint = FALSE;
					*result = CDRF_SKIPDEFAULT;
				}
				else 
				{
					temp2<<"prepaint ";
					debugClient.SendEvent("CustomDraw",temp2);
					*result=CDRF_NOTIFYITEMDRAW;
				}
				return TRUE;

			case CDDS_POSTPAINT:
				temp2<<"postpaint "<<temp;
				debugClient.SendEvent("CustomDraw",temp2);
				if (m_tree) m_tree->InvalidateRect();
				return TRUE;

			case CDDS_ITEMPREPAINT:
				bIsHighlighted = FALSE;
				iRow = cd->nmcd.dwItemSpec;
				if (IsLineSelected(iRow))
				{
					if (IsFocus())
					{
						bIsHighlighted = TRUE;
						cd->clrText   = m_textColorHi; // Use my foreground color
						cd->clrTextBk = m_backgroundColorHi; // Use my background color
						EnableLineSelected(iRow, false);
					}	
					/*else
					{
						bIsHighlighted = TRUE;
						cd->clrText   = m_textColorHiUf; // Use my foreground color
						cd->clrTextBk = m_backgroundColorHiUf; // Use my background color
						EnableLineSelected(iRow, false);
					}	*/
				}
				else
				{
					cd->clrText   = m_textColor; // Use my foreground color
					cd->clrTextBk = m_backgroundColor; // Use my background color
				}
				*result = CDRF_NOTIFYPOSTPAINT;
				temp2<<"item prepaint - bIsHL:"<<bIsHighlighted<<" iRaw:"<<iRow;
				debugClient.SendEvent("CustomDraw",temp2);
				return TRUE;

			case CDDS_ITEMPOSTPAINT:
				iRow = cd->nmcd.dwItemSpec;
				temp2<<"item postpaint - bIsHL:"<<bIsHighlighted<<" iRow:"<<iRow;
				debugClient.SendEvent("CustomDraw",temp2);

				if (bIsHighlighted)
				{
					// Turn listview control's highlighting back on now that we have
					// drawn the row in the colors we want.
					EnableLineSelected(iRow, true);
					isRepaint = TRUE;
				}

				*result = CDRF_DODEFAULT;
				return TRUE;
			default:
				temp2<<"DEFAULT "<<temp;
				debugClient.SendEvent("CustomDraw",temp2);
				*result = CDRF_DODEFAULT;
				return TRUE;
			}
			break;
	}
	return FALSE;
}

// *********************************************************************************

int CMyListView::GetLinesSelected(CMyList<int> *lstSelect)
{ 
	int index = -1;
	lstSelect->SuprAll();
	do
	{
		index = ListView_GetNextItem(GetHWND(), index, LVNI_SELECTED);
		if (index!=-1) lstSelect->AddLast(index);
	} while (index!=-1);
	return lstSelect->GetNbElem();
}

// *********************************************************************************

void CMyListView::SetLineSelected(int index,BOOL alwaysOneSelected)
{
	if (alwaysOneSelected)
	{
		if (index==-1) index = min(m_lineSelected,m_nbCount-1);
		else { if (index>=m_nbCount) index = m_nbCount-1; }
	}
	
	ListView_SetItemState (GetHWND(),-1,0,LVIS_SELECTED);
	ListView_SetItemState (GetHWND(),index,LVIS_SELECTED,LVIS_SELECTED);
	m_lineSelected = index;
}

// *****************************************************************************

BOOL CMyListView::SetItemHeight(int height)
{
	// change la hauteru de la list
	HIMAGELIST himl;
	himl = ImageList_Create(1,height,ILC_COLOR4,1,1);
	HICON hIcon;
	BYTE *dataBit = new BYTE[height];
	hIcon = CreateIcon(GethInstance(),1,height,1,1,dataBit,dataBit);
	delete dataBit;
	ImageList_AddIcon(himl,hIcon);
	return (BOOL) ListView_SetImageList(GetHWND(),himl,LVSIL_SMALL);
}

// *****************************************************************************

int CMyListView::GetColumnWidth(int index)
{
	return ListView_GetColumnWidth(m_hWndControl,index);
}

// ***************************************************************************************

BOOL CMyListView::SetImageList (HIMAGELIST himl)
{
	return (BOOL)ListView_SetImageList(m_hWndControl,himl,LVSIL_SMALL);
}

// ***************************************************************************************

BOOL CMyListView::GetFullRawSelect()
{
	return (ListView_GetExtendedListViewStyle(GetHWND()) & LVS_EX_FULLROWSELECT) == LVS_EX_FULLROWSELECT;
}

// ***************************************************************************************

BOOL CMyListView::SetFullRawSelect(BOOL set)
{
	return ListView_SetExtendedListViewStyleEx(GetHWND(),LVS_EX_FULLROWSELECT,set?LVS_EX_FULLROWSELECT:0);
}

// ***************************************************************************************

BOOL CMyListView::SetCheckBox(BOOL set)
{
	return ListView_SetExtendedListViewStyleEx(GetHWND(),LVS_EX_CHECKBOXES,set?LVS_EX_CHECKBOXES:0);
}

// ***************************************************************************************

BOOL CMyListView::SetHeaderDragDrop(BOOL set)
{
	return ListView_SetExtendedListViewStyleEx(GetHWND(),LVS_EX_HEADERDRAGDROP,set?LVS_EX_HEADERDRAGDROP:0);
}

// ***************************************************************************************

BOOL CMyListView::SetSubItemImages(BOOL set)
{
	return ListView_SetExtendedListViewStyleEx(GetHWND(),LVS_EX_SUBITEMIMAGES,set?LVS_EX_SUBITEMIMAGES:0);
}

// ***************************************************************************************

BOOL CMyListView::SetTrackSelect(BOOL set)
{
	return ListView_SetExtendedListViewStyleEx(GetHWND(),LVS_EX_TRACKSELECT,set?LVS_EX_TRACKSELECT:0);
}
 
// ***************************************************************************************

HWND CMyListView::GetHeader()
{
	return ListView_GetHeader(GetHWND());
}

// ***************************************************************************************

void CMyListView::SetHightLightColor(COLORREF textColor,COLORREF backgroundColor)
{
	m_textColorHi = textColor;
	m_backgroundColorHi = backgroundColor;
	if (m_tree) m_tree->SetHightLightColor(textColor,backgroundColor);
}

// ***************************************************************************************

void CMyListView::GetHightLightColor(COLORREF *textColor,COLORREF *backgroundColor)
{
	if (textColor) *textColor = m_textColorHi;
	if (backgroundColor) *backgroundColor= m_backgroundColorHi;
}

// ***************************************************************************************

void CMyListView::SetHightLightColorUnFocus(COLORREF textColor,COLORREF backgroundColor)
{
	m_textColorHiUf = textColor;
	m_backgroundColorHiUf = backgroundColor;
	if (m_tree) m_tree->SetHightLightColorUnFocus(textColor,backgroundColor);
}

// ***************************************************************************************

void CMyListView::GetHightLightColorUnFocus(COLORREF *textColor,COLORREF *backgroundColor)
{
	if (textColor) *textColor = m_textColorHiUf;
	if (backgroundColor) *backgroundColor= m_backgroundColorHiUf;
}

// ***************************************************************************************

void CMyListView::SetLineColor(COLORREF textColor,COLORREF backgroundColor)
{
	m_textColor = textColor;
	m_backgroundColor = backgroundColor;
	if (m_tree) m_tree->SetLineColor(textColor,backgroundColor);
}

// ***************************************************************************************

void CMyListView::GetLineColor(COLORREF *textColor,COLORREF *backgroundColor)
{
	if (textColor) *textColor = m_textColor;
	if (backgroundColor) *backgroundColor= m_backgroundColor;
}

// ***************************************************************************************

void CMyListView::FirstCollumnAsTree(CMyTree *tree)
{
	m_tree=tree;
	m_tree->SetPos(2,5+GetSystemMetrics(SM_CYHSCROLL),GetHWND());
	m_tree->SetFullRawSelect(TRUE);
	
	m_tree->SetLineColor(m_textColor,m_backgroundColor);
	m_tree->SetHightLightColor(m_textColorHi,m_backgroundColorHi);
	m_tree->SetHightLightColorUnFocus(m_textColorHiUf,m_backgroundColorHi);

	SetItemHeight(m_tree->GetItemHeight()-1);
	TreeColumnSizeChanged();
}

// ***************************************************************************************

void CMyListView::TreeColumnSizeChanged()
{
	int dy;
	int dx = GetColumnWidth(0)-1;
	GetSize(NULL,&dy);
	dy -= GetSystemMetrics(SM_CYHSCROLL)+3;
	m_tree->SetSize(dx+150,dy);
	HRGN hRgn = CreateRectRgn(0,0,0,0);
	int dyItem = m_tree->GetItemHeight();
	for (int i=0;i<dy;i+=dyItem)
	{
		HRGN hRgnCopy = CreateRectRgn(0,0,100,100);
		CombineRgn(hRgnCopy,hRgn,NULL,RGN_COPY);
		HRGN hRgnCur = CreateRectRgn(1,i,dx,i+dyItem-1);
		CombineRgn(hRgn,hRgnCopy,hRgnCur,RGN_OR);
		DeleteObject(hRgnCopy);
		DeleteObject(hRgnCur);
	}
	SetWindowRgn(*m_tree,hRgn,FALSE);
	DeleteObject(hRgn);
}

// ***************************************************************************************

void CMyListView::UnSelectedAll() 
{ 
	ListView_SetItemState (GetHWND(),-1,0,LVIS_SELECTED); 
}

// ***************************************************************************************

void CMyListView::EnableLineSelected (int index,BOOL set) 
{ 
	ListView_SetItemState (GetHWND(),index,set?0xff:0,LVIS_SELECTED); 
	//ListView_SetItemState (GetHWND(),index,set?LVIS_SELECTED:0,LVIS_SELECTED); 
}

// ***************************************************************************************

BOOL CMyListView::IsLineSelected (int index) 
{ 
	return ListView_GetItemState (GetHWND(),index,LVIS_SELECTED) != 0; 
}

// ***************************************************************************************

void CMyListView::SetHotItem (int index) 
{ 
	ListView_SetHotItem(GetHWND(),index); 
}

// ***************************************************************************************

void CMyListViewNotify::OnColumnSizeChanged(int numColumn,int newSize)
{
#ifdef DEBUG_LISTVIEW
	titleDebug="";
	titleDebug<<"HWND:"<<(long)hWnd;
	textDebug="ListView_OnColumnSizeChanged = ";
	textDebug<<"NumColumn:"<<numColumn<<" , newSize:"<<newSize;
	debug.SendEvent(titleDebug,textDebug);
#endif
}

void CMyListViewNotify::OnGetText(int line,int column,char *text)
{
#ifdef DEBUG_LISTVIEW
	titleDebug="";
	titleDebug<<"HWND:"<<(long)hWnd;
	textDebug="ListView_OnGetText = ";
	textDebug<<"line:"<<line<<" , column:"<<column;
	strcpy(text,textDebug); // pour debug

	debug.SendEvent(titleDebug,textDebug);
#endif
}

void CMyListViewNotify::OnItemChanged(int line,int column)
{
#ifdef DEBUG_LISTVIEW
	titleDebug="";
	titleDebug<<"HWND:"<<(long)hWnd;
	textDebug="ListView_OnItemChanged = ";
	textDebug<<"line:"<<line<<" , column:"<<column;
	debug.SendEvent(titleDebug,textDebug);
#endif
}

void CMyListViewNotify::OnKeyDown(WORD vKey)
{
#ifdef DEBUG_LISTVIEW
	titleDebug="";
	titleDebug<<"HWND:"<<(long)hWnd;
	textDebug="ListView_OnKeyDown = ";
	textDebug<<"vKey:"<<vKey;
	debug.SendEvent(titleDebug,textDebug);
#endif
}
