#include "stdafx.h"
#include "MyRichEdit.h"
#include "MyControl.h"

#include "richedit.h"
#include <windowsx.h>
#include <commctrl.h> // pour InitCommonControls

// ***************************************************************************************

LRESULT CALLBACK RichEditCallBack (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	CMyRichEdit *edit= (CMyRichEdit *) GetWindowLong(hDlg,GWL_USERDATA);
	if (!edit) return TRUE; // WM_DESTROY
/*
	switch (message)
	{
		case WM_MOUSEMOVE:
			if (!edit->IsFocus()) edit->SetFocus();
			break;
	}
*/
	return edit->sub.CallWindowProc(message,wParam,lParam);
}

// ***************************************************************************************

LRESULT CALLBACK RichEditParentCallBack (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	CMyRichEdit *edit = (CMyRichEdit *) GetWindowLong(hDlg,GWL_USERDATA);
	if (!edit) return FALSE; // WM_DESTROY 
	switch (message)
	{ 
		case WM_NOTIFY:
			{
				ENotifyRichEdit eNotify=edit->Notify(lParam);	
				if (eNotify!=RichEditNothing) 
				{
					//PostMessage(edit->GetParent(),WM_COMMAND,edit->GetControlID(),eNotify);
					return edit->subParent.CallWindowProc(WM_COMMAND,edit->GetControlID(),eNotify);
					break;
				}
			}
			break;

		case WM_COMMAND:
			if ((HWND)lParam!=edit->GetHWND()) break;

			switch (HIWORD(wParam))
			{
				case EN_CHANGE:
					edit->Refresh(FALSE);
					//PostMessage(edit->GetParent(),WM_COMMAND,edit->GetControlID(),RichEditChange);
					return edit->subParent.CallWindowProc(WM_COMMAND,edit->GetControlID(),RichEditChange);
			}
			break;
	}
	return edit->subParent.CallWindowProc(message,wParam,lParam);
}

// **********************************************************************************

CMyRichEdit::CMyRichEdit() : CMyControl()
{
	m_controlID = 0;
	Init();
}

CMyRichEdit::CMyRichEdit(HWND hWnd) : CMyControl(hWnd)
{
	Init();
	ReInit();	
}

// **********************************************************************************

CMyRichEdit::CMyRichEdit(HWND hwndMaster,int x,int y,int dx,int dy,int optionalControlID) 
: CMyControl(hwndMaster)
{
	InitCommonControls();

	m_controlID = optionalControlID;

	DWORD dwStyle = WS_CHILD | WS_BORDER | WS_VISIBLE;
	m_hWndControl = CreateWindowEx( 
		WS_EX_CLIENTEDGE,RICHEDIT_CLASS,"",dwStyle,
		x,y,dx,dy,
		hwndMaster,NULL,GethInstance(),NULL);

	Init();
	ReInit();
}

// **********************************************************************************

void CMyRichEdit::Init()
{
	m_isBold = FALSE;
	m_isItalic = FALSE;
	m_isUnderline = FALSE;
	m_color =  GetSysColor (COLOR_WINDOWTEXT);
	m_isAutoScroll = FALSE;
	m_isFormatedToFileNane = FALSE;
}

// **********************************************************************************

void CMyRichEdit::ReInit()
{
	// enable la reception des messages WM_NOTIFY et WM_COMMAND suivante:
	SendMessage(EM_SETEVENTMASK,NULL,ENM_DRAGDROPDONE|ENM_CHANGE|ENM_DROPFILES|ENM_KEYEVENTS); 

	subParent.Set(RichEditParentCallBack,(long)this);
	//sub.Set(RichEditCallBack);
	Refresh(TRUE);
}

// **********************************************************************************

BOOL CMyRichEdit::operator=(HWND hWnd)
{
	CMyControl *control = (CMyControl *)this;
	*control = hWnd;
	m_controlID = GetDlgCtrlID();
	ReInit();
	return TRUE;
}

// **********************************************************************************

void CMyRichEdit::RemplaceSelection(char *textCur)
{
	GetText(text);
	ApplyEffect();
	SendMessage(EM_REPLACESEL,FALSE,(LPARAM) textCur);
	if (m_isFormatedToFileNane) FormatedToFileNane();
	if (m_isAutoScroll) scroll.Bottom();
}

void CMyRichEdit::InsertText(char *textCur,int pos)
{
	if (pos!=-1) SetSelection(pos,pos);
	else SetSelection(-1,-1);
	RemplaceSelection(textCur);
}

// **********************************************************************************

void CMyRichEdit::SetSelectionAll()
{
	SetSelection(0,-1);
}

void CMyRichEdit::SetSelection(int posBegin,int posEnd)
{
	Edit_SetSel(GetHWND(),posBegin,posEnd);
}

// **********************************************************************************

void CMyRichEdit::GetSelection(int *posBegin,int *posEnd)
{
	SendMessage(EM_GETSEL,(WPARAM)posBegin,(LPARAM)posEnd);
}

// **********************************************************************************

void CMyRichEdit::SetEffect(BOOL isBold,BOOL isItalic,BOOL isUnderline,COLORREF color)
{
	if (isBold!=-1) m_isBold = isBold;
	if (isItalic!=-1) m_isItalic = isItalic;
	if (isUnderline!=-1) m_isUnderline = isUnderline;
	if (color!=-1) m_color = color;
}

// **********************************************************************************

void CMyRichEdit::ApplyEffect()
{
	CHARFORMAT pChrFormat ;
	pChrFormat.cbSize = sizeof (CHARFORMAT) ;
	pChrFormat.dwMask = CFM_COLOR|CFM_BOLD|CFM_ITALIC|CFM_UNDERLINE;
	pChrFormat.dwEffects = (m_isBold?CFM_BOLD:0)|(m_isItalic?CFM_ITALIC:0)|(m_isUnderline?CFM_UNDERLINE:0);
	pChrFormat.crTextColor = m_color;
	SendMessage (EM_SETCHARFORMAT,SCF_SELECTION,(LPARAM)&pChrFormat);
}

// **********************************************************************************

void CMyRichEdit::operator=(char *textCur) 
{
	text=textCur;
	Refresh(TRUE);
}

// **********************************************************************************

void CMyRichEdit::Refresh(BOOL isActif) 
{
	if (isActif) 
	{
		if (!m_isFormatedToFileNane) ApplyEffect();
		SetText(text);
	}
	else
	{
		GetText(text);
	}

	if (m_isFormatedToFileNane) FormatedToFileNane();
	if (m_isAutoScroll) scroll.Bottom();
}

// **********************************************************************************

void CMyRichEdit::SetAutoScroll(BOOL isAutoScroll)
{
	m_isAutoScroll = isAutoScroll;
	scroll.Bottom();
}

// ********************************************************************************** 

void CMyRichEdit::FormatedToFileNane()
{
	int posBegin,posEnd;
	GetSelection(&posBegin,&posEnd);
	
	SetSelection(0,text.size);
	SetEffect(FALSE,FALSE,FALSE,RGB(0,0,0));
	ApplyEffect();
	
	for (int i=0;i<text.size;i++)
	{
		switch (text[i])
		{
			case ':':
			case '\\':
			case '.':
				SetSelection(i,i+1);
				SetEffect(TRUE,FALSE,FALSE,RGB(0,64,128));
				ApplyEffect();
				break;
		}
	}
	SetSelection(posBegin,posEnd);
}

// ********************************************************************************** 

ENotifyRichEdit CMyRichEdit::Notify (LPARAM lParam)
{
	LPNMHDR lpnmh = (LPNMHDR) lParam;
	ENotifyRichEdit eNotify = RichEditNothing;

	if (lpnmh->hwndFrom != GetHWND()) return eNotify;

	MyString temp;

	switch (lpnmh->code)
	{
		case EN_MSGFILTER:
			{
				MSGFILTER *mf = (MSGFILTER *) lParam;
				switch (mf->msg)
				{
					case WM_KEYUP:
						eNotify=RichEditKeyUp;
					    m_lastKeyUp = mf->wParam;
						break;
				}
			}
			break;
			
		case EN_DROPFILES:
		case EN_DRAGDROPDONE:
			break;
	}
	return eNotify;
}

// **********************************************************************************

// **********************************************************************************
// **********************************************************************************
