#include <windows.h>
#include <stdio.h>
#include <wingdi.h>

#include "MyScreen.h"

// ***************************************************************************************

CMyScreen::CMyScreen()
{
	m_buffer = NULL;
	m_heigth = 0;
	m_width = 0;
}

// ***************************************************************************************

CMyScreen::~CMyScreen()
{
	if (m_buffer) delete []m_buffer;
}

/* ***************************************************************************************
	Pour accelerer les calculs (en simplifiant la formul + precalculation)

    //equation d'origine
	int index = x*3+(m_heigth-y-1)*m_width*3; 
*************************************************************************************** */

void CMyScreen::PreCompute()
{
	m_heigth--; 
	m_sum = m_heigth*m_width;
}

// ***************************************************************************************

BOOL CMyScreen::CopyToBuffer_Slow(HDC hDC,int dx,int dy)
{
	m_heigth = dx; 
	m_width = dy;
	PreCompute();
	if (!m_buffer) m_buffer = new UCHAR[m_sum*3];
	if (!m_buffer) return FALSE;

	for (int y=0;y<m_heigth+1;y++) // car precompute retir 1
		for (int x=0;x<m_width;x++)
			SetPixel(GetPixelIndex(x,y),::GetPixel(hDC,x,y));
	return TRUE;
}

BOOL CMyScreen::CopyToBuffer(HWND hWnd)
{
	HDC hDC;
	if (!hWnd) 
	{
		hWnd= GetDesktopWindow();
		hDC = GetDC(NULL);
	}
	else hDC = GetDC(hWnd);
	RECT WndRect;
	GetWindowRect(hWnd,&WndRect);
	int dy = WndRect.bottom - WndRect.top; 
	int dx = WndRect.right - WndRect.left;
	BOOL returnValue = CopyToBuffer(hDC,dx,dy);
	ReleaseDC(hWnd,hDC);
	return returnValue;
}

BOOL CMyScreen::CopyToBuffer(HDC hDC,int dx,int dy)
{
	m_heigth = dy;
	m_width = dx;

	HDC hMemDC = CreateCompatibleDC(hDC);
	HBITMAP  hWndBitmap = CreateCompatibleBitmap(hDC,m_width,m_heigth); // si hMemDC ==> Monocrom
	BITMAP WndBmp;
	GetObject(hWndBitmap,sizeof(WndBmp),&WndBmp);
	SelectObject(hMemDC,hWndBitmap);

	BitBlt(hMemDC,0,0,m_width,m_heigth,hDC,0,0,SRCCOPY);

	BITMAPINFOHEADER bi;
	ZeroMemory(&bi,sizeof(BITMAPINFOHEADER));
	bi.biSize               = sizeof(BITMAPINFOHEADER);
	bi.biWidth              = WndBmp.bmWidth;
	bi.biHeight             = WndBmp.bmHeight;
	bi.biPlanes             = WndBmp.bmPlanes; //1;
	bi.biBitCount           = 24;

	GetDIBits (hDC,hWndBitmap,0,WndBmp.bmHeight,NULL,(LPBITMAPINFO)&bi,DIB_RGB_COLORS);
	if (!m_buffer) m_buffer = new UCHAR[bi.biSizeImage];
	if (!m_buffer) return FALSE;

	GetDIBits (hDC,hWndBitmap,0,WndBmp.bmHeight,m_buffer,(LPBITMAPINFO)&bi,DIB_RGB_COLORS);

	DeleteDC(hMemDC);
	DeleteObject(hWndBitmap);

	PreCompute();
	return TRUE;
}

// ***************************************************************************************

BOOL CMyScreen::CopyToScreen()
{
	HWND hWnd = GetDesktopWindow();// Rcupration du DC de la fentre
	HDC hWndDC = GetDC(NULL);

	RECT WndRect;
	GetWindowRect(hWnd,&WndRect);
	m_heigth = WndRect.bottom - WndRect.top ;
	m_width = WndRect.right - WndRect.left;

	HDC hMemDC = CreateCompatibleDC(hWndDC);
	HBITMAP  hWndBitmap = CreateCompatibleBitmap(hWndDC,m_width,m_heigth); // si hMemDC ==> Monocrom
	BITMAP WndBmp;
	GetObject(hWndBitmap,sizeof(WndBmp),&WndBmp);
	SelectObject(hMemDC,hWndBitmap);

	BITMAPINFOHEADER bi;
	ZeroMemory(&bi,sizeof(BITMAPINFOHEADER));
	bi.biSize               = sizeof(BITMAPINFOHEADER);
	bi.biWidth              = WndBmp.bmWidth;
	bi.biHeight             = WndBmp.bmHeight;
	bi.biPlanes             = WndBmp.bmPlanes; //1;
	bi.biBitCount           = 24;

	SetDIBits (hWndDC,hWndBitmap,0,WndBmp.bmHeight,m_buffer,(LPBITMAPINFO)&bi,DIB_RGB_COLORS);

	BitBlt(hWndDC,0,0,m_width,m_heigth,hMemDC,0,0,SRCCOPY);

	DeleteDC(hMemDC);
	DeleteObject(hWndBitmap);

	PreCompute();
	return TRUE;
}

// ***************************************************************************************

COLORREF CMyScreen::GetPixel(int x, int y)
{
	int index = (x+m_sum-y*m_width)*3;
	return RGB(m_buffer[index+2],m_buffer[index+1],m_buffer[index]);
}

// ***************************************************************************************

int CMyScreen::GetPixelIndex(int x, int y) 
{ 
	return (x+m_sum-y*m_width)*3; 
}

// ***************************************************************************************

COLORREF CMyScreen::GetPixel(int index) 
{ 
	return RGB(m_buffer[index+2],m_buffer[index+1],m_buffer[index]); 
}

// ***************************************************************************************

void CMyScreen::SetPixel(int index,COLORREF color) 
{ 
	m_buffer[index+2]=GetRValue(color);
	m_buffer[index+1]=GetGValue(color);
	m_buffer[index]=GetBValue(color);
}

// ***************************************************************************************

void CMyScreen::SetPixel(int index,UCHAR red,UCHAR green,UCHAR blue) 
{ 
	m_buffer[index+2]=red;
	m_buffer[index+1]=green;
	m_buffer[index]=blue;
}

// ***************************************************************************************

void CMyScreen::SetPixel(int x, int y,UCHAR red,UCHAR green,UCHAR blue)
{
	int index = (x+m_sum-y*m_width)*3;
	m_buffer[index+2]=red;
	m_buffer[index+1]=green;
	m_buffer[index]=blue;
}

// ***************************************************************************************

void CMyScreen::SetPixel(int x, int y,COLORREF color)
{
	int index = (x+m_sum-y*m_width)*3;
	m_buffer[index+2]=GetRValue(color);
	m_buffer[index+1]=GetGValue(color);
	m_buffer[index]=GetBValue(color);
}

// ***************************************************************************************
